app.component('tab-seo', {
	props: ['item', 'formData', 'formDefinitions', 'saved'],
	data: function(){
		return {
			seotitleextended : '',
			seotitle : '',
			seodescription : '',
			seodate : '',
			titleLength : 65,
			descriptionLength: 160,
			disabled: false,
			message: false,
			messageClass: '',
			errors: {}
		}
	},
	template: `<section class="dark:bg-stone-700 dark:text-stone-200">
					<form class="inline-block w-full mb-8">
						<div class="w-full">
							<fieldset class="w-full border border-stone-200 px-4 py-8">
								<legend class="text-lg font-medium">Preview</legend>
								<div class="flex items-center justify-center flex-col">
									<div class="w-full">
										<h3 style="color:#1a0dab;font-size:20px;line-height:25px;padding-top:5px;margin-bottom:3px;text-transform:none;">{{ seotitle }}{{ seotitleextended }}</h3>
										<p>{{ seodescription }}</p>
									</div>
									<div class="w-full mt-4 flex text-xs">
										<span class="p-2 mr-2" :class="getTitleAlert()">Title: {{ countTitleChars() }} ({{titleLength}})</span>
										<span class="p-2" :class="getDescriptionAlert()">Description: {{ countDescChars() }} ({{descriptionLength}})</span>
									</div>
								</div>
							</fieldset>
						</div>
						<div class="w-full">
							<fieldset class="flex flex-wrap justify-between border-2 border-stone-200 p-4 my-8">
								<legend class="text-lg font-medium">Meta-Content</legend>
								<div class="w-full mt-5 mb-5">
									<label for="title" class="block mb-1 font-medium">Meta title</label>
									<input 
										type 		= "text" 
										v-model 	= "seotitle"										
										class 		= "text-stone-900 h-12 w-full border px-2 py-3 border-stone-300 bg-stone-200" 
										maxlength 	= "100" 
										name 		= "title"
									>
									<span v-if="errors.title" class="bg-rose-500 text-white text-xs p-1">{{errors.title[0]}}</span>
								</div>
								<div class="w-full mt-5 mb-5" size="160">
									<label for="description" class="block mb-1 font-medium">Meta description</label>
									<textarea 
										v-model ="seodescription"									
										rows 	= "4" 
										class 	= "w-full border border-stone-300 text-stone-900 bg-stone-200 px-2 py-3 border-stone-300 bg-stone-200" 
										name 	= "description"
									></textarea>
									<span v-if="errors.description" class="bg-rose-500 text-white text-xs p-1">{{errors.description[0]}}</span>
								</div>
							</fieldset>
						</div>
						<div class="my-5">
							<div class="block w-full h-8 my-1">
								<transition name="fade">
									<div v-if="message" :class="messageClass" class="text-white px-3 py-1 transition duration-100">{{ $filters.translate(message) }}</div>
								</transition>
							</div>
							<input type="submit" :disabled="disabled" @click.prevent="saveInput()" :value="$filters.translate('save meta')" class="w-full p-3 my-1 bg-stone-700 dark:bg-stone-600 hover:bg-stone-900 hover:dark:bg-stone-900 text-white cursor-pointer disabled:cursor-not-allowed disabled:bg-stone-200 disabled:text-stone-900 transition duration-100">
						</div>                      
					</form>
					<gscdata :item="item"></gscdata>
				</section>`,
	mounted: function()
	{
		this.seotitleextended 	= titleAddition;
		this.seotitle 			= this.$parent.formData['meta']['title'];
		this.seodescription 	= this.$parent.formData['meta']['description'];
		this.seodate 			= this.getMetaDate();
	},
	methods: {
		getMetaDate: function()
		{
			if(this.$parent.formData['meta']['manualdate'])
			{
				return this.$parent.formData['meta']['manualdate'];
			}
			else if(this.$parent.formData['meta']['modified'])
			{
				return this.$parent.formData['meta']['modified'];
			}
			return this.$parent.formData['meta']['created'];
		},
		getTitleAlert: function()
		{
			var titleLength = (this.seotitle.length + this.seotitleextended.length);
			if(titleLength > this.titleLength)
			{
				return 'bg-rose-500 text-white';
			}
			else if(titleLength > this.titleLength-5)
			{
				return 'bg-yellow-500';
			}
			return 'bg-stone-200';
		},
		getDescriptionAlert: function()
		{
			var descriptionLength = (this.seodescription.length + this.seodate.length);
			if(descriptionLength > this.descriptionLength)
			{
				return 'bg-rose-500 text-white';
			}
			else if(descriptionLength > this.descriptionLength-5)
			{
				return 'bg-yellow-500';
			}
			return 'bg-stone-200';
		},
		countDescChars: function()
		{
			return (this.seodescription.length + this.seodate.length);
		},
		countTitleChars: function()
		{
			return (this.seotitle.length + this.seotitleextended.length);
		},
		selectComponent: function(type)
		{ 
			return 'component-' + type;
		},
/*
		saveInput: function()
		{
			this.$emit('saveform');
		},
*/
		saveInput: function()
		{
			this.saved = false;
			this.errors = {};
			this.disabled = true;
			this.message = false;
			this.messageClass = 'bg-stone-50';

			var selfmeta = this;

			$data = {
				'title':  this.seotitle, 
				'description': this.seodescription
			};

			tmaxios.post('/api/v1/seometa',{
				'url':			data.urlinfo.route,
				'data':  		$data,
				'item': 		this.item
			})
			.then(function (response) {
				selfmeta.disabled 		= false;
				selfmeta.saved 			= true;
				selfmeta.message 		= 'saved successfully';
				selfmeta.messageClass 	= 'bg-teal-500';				
				
				self.$parent.formData['meta']['title'] = selfmeta.seotitle;
				self.$parent.formData['meta']['description'] = selfmeta.seodescription;
			})
			.catch(function (error)
			{
				selfmeta.disabled = false;
				
				if(error.response)
				{
					let errors = error.response.data.errors;
					let message = handleErrorMessage(error);

					if(message)
					{				
						selfmeta.messageClass 	= 'bg-rose-500';
						selfmeta.message 		= message;
					}

					if(errors)
					{
						selfmeta.errors = errors;
					}
				}
			});			
		}
	}
})


app.component('gscdata', {
	props: ['item'],
	data: function(){
		return {
			disabled: true,
			gscdata: false,
			serviceaccount: true,
			serviceaccountmessage: false,
			keyfile: '',
			errors: false,
			country: '',
			countrylist: ['ABW','AFG','AGO','AIA','ALA','ALB','AND','ARE','ARG','ARM','ASM','ATA','ATF','ATG','AUS','AUT','AZE','BDI','BEL','BEN','BES','BFA','BGD','BGR','BHR','BHS','BIH','BLM','BLR','BLZ','BMU','BOL','BRA','BRB','BRN','BTN','BVT','BWA','CAF','CAN','CCK','CHE','CHL','CHN','CIV','CMR','COD','COG','COK','COL','COM','CPV','CRI','CUB','CUW','CXR','CYM','CYP','CZE','DEU','DJI','DMA','DNK','DOM','DZA','ECU','EGY','ERI','ESH','ESP','EST','ETH','FIN','FJI','FLK','FRA','FRO','FSM','GAB','GBR','GEO','GGY','GHA','GIB','GIN','GLP','GMB','GNB','GNQ','GRC','GRD','GRL','GTM','GUF','GUM','GUY','HKG','HMD','HND','HRV','HTI','HUN','IDN','IMN','IND','IOT','IRL','IRN','IRQ','ISL','ISR','ITA','JAM','JEY','JOR','JPN','KAZ','KEN','KGZ','KHM','KIR','KNA','KOR','KWT','LAO','LBN','LBR','LBY','LCA','LIE','LKA','LSO','LTU','LUX','LVA','MAC','MAF','MAR','MCO','MDA','MDG','MDV','MEX','MHL','MKD','MLI','MLT','MMR','MNE','MNG','MNP','MOZ','MRT','MSR','MTQ','MUS','MWI','MYS','MYT','NAM','NCL','NER','NFK','NGA','NIC','NIU','NLD','NOR','NPL','NRU','NZL','OMN','PAK','PAN','PCN','PER','PHL','PLW','PNG','POL','PRI','PRK','PRT','PRY','PSE','PYF','QAT','REU','ROU','RUS','RWA','SAU','SDN','SEN','SGP','SGS','SHN','SJM','SLB','SLE','SLV','SMR','SOM','SPM','SRB','SSD','STP','SUR','SVK','SVN','SWE','SWZ','SXM','SYC','SYR','TCA','TCD','TGO','THA','TJK','TKL','TKM','TLS','TON','TTO','TUN','TUR','TUV','TWN','TZA','UGA','UKR','UMI','URY','USA','UZB','VAT','VCT','VEN','VGB','VIR','VNM','VUT','WLF','WSM','YEM','ZAF','ZMB','ZWE'],
			startdate: '',
			enddate: '',
		 	jsonPlaceholder: `{
			  "type": "service_account",
			  "project_id": "myid",
			  "private_key_id": "4444a11111t3333g4444b1111t12345",
			  "private_key": "-----BEGIN PRIVATE KEY-----XXX",
			  "client_email": "727272727272727272-compute@developer.gserviceaccount.com",
			  "client_id": "10101010101010fffff",
			  "auth_uri": "https://accounts.google.com/o/oauth2/auth",
			  "token_uri": "https://oauth2.googleapis.com/token",
			  "auth_provider_x509_cert_url": "https://www.googleapis.com/oauth2/v1/certs",
			  "client_x509_cert_url": "https://www.googleapis.com/robot/v1/metadata/x509/..." \n}`
		}
	},
	template: `<div class="inline-block pt-2 mt-8 bg-stone-100 p-6">
					<h2 class="text-3xl font-bold mt-6 mb-4">Google Search Console Data</h2>
					<p>Display search data from the google search console and get detailed search-insights for this content page.</p>
					<div v-if="!serviceaccount">
						<p class="mt-2">To get the data from the google-api, a service-account.json-file is required. Read all about it on the <a href="https://plugins.typemill.net/seo" class="link text-teal-500">plugin page.</a></p>
						<form>
							<div class="w-full mt-5 mb-5" size="160">
								<label for="description" class="block mb-1 font-medium">Add your google service account key</label>
								<textarea 
									v-model ="keyfile"
									:placeholder="jsonPlaceholder"
									rows 	= "14" 
									class 	= "w-full border border-stone-300 text-stone-900 bg-stone-200 px-2 py-3 border-stone-300 bg-stone-200" 
									name 	= "keyfile"
								></textarea>
								<div class="flex justify-between mt-1">
									<div class="w-3/4 bg-stone-100 mr-1">
										<div v-if="serviceaccountmessage" class="bg-rose-500 text-white p-2">{{ serviceaccountmessage }}</div>
										<div v-else class="p-2">Add valid service-account-data and save.</div>
									</div>
									<input type="submit" :disabled="disabled" class="w-1/4 center text-white px-4 py-2 bg-stone-900 cursor-pointer hover:bg-stone-700 disabled:cursor-not-allowed disabled:bg-stone-200 disabled:text-stone-900" @click.prevent="saveServiceAccountKey" value="save key"></input>
								</div>
							</div>
						</form>
					</div>
					<div v-if="gscdata">
						<form class="mt-6 mb-4 flex justify-between" style="align-items:flex-end;">
							<div class="w-1/4 mr-2">
								<label for="country" class="block mb-1 font-medium">Country</label>
								<select name="country" class="w-full h-12 border px-2 py-3 border-stone-300 bg-stone-200 text-stone-900" v-model="country">
									<option disabled value="">Please select one</option>
									<option value=""></option>
									<option v-for="countrycode in countrylist">{{countrycode}}</option>
								</select>
							</div>
							<div class="w-1/4 mr-2">
								<label for="startdate" class="block mb-1 font-medium">Startdate</label>
								<input name="startdate" class="w-full h-12 border px-2 py-3 border-stone-300 bg-stone-200 text-stone-900" type="date" v-model="startdate">
							</div>
							<div class="w-1/4 mr-2">
								<label for="enddate" class="block mb-1 font-medium">Enddate</label>
								<input name="enddate" class="w-full h-12 border px-2 py-3 border-stone-300 bg-stone-200 text-stone-900" type="date" v-model="enddate">
							</div>
							<button :disabled="disabled" class="w-1/4 px-2 py-3 ml-2 text-stone-50 bg-stone-700 hover:bg-stone-900 hover:text-white transition duration-100 cursor-pointer disabled:cursor-not-allowed disabled:bg-stone-200 disabled:text-stone-900" @click.prevent="getDataFromGSC">update</button>
						</form>
						<div class="w-full overflow-auto">
							<table class="w-full mt-8 collapse">
								<thead>
									<tr>
										<th class="p-3 bg-stone-200 dark:bg-stone-900 border-2 border-stone-50 dark:border-stone-600">Query</th>
										<th class="p-3 bg-stone-200 dark:bg-stone-900 border-2 border-stone-50 dark:border-stone-600">Clicks</th>
										<th class="p-3 bg-stone-200 dark:bg-stone-900 border-2 border-stone-50 dark:border-stone-600">Impressions</th>
										<th class="p-3 bg-stone-200 dark:bg-stone-900 border-2 border-stone-50 dark:border-stone-600">CTR</th>
										<th class="p-3 bg-stone-200 dark:bg-stone-900 border-2 border-stone-50 dark:border-stone-600">Position</th>
									</tr>
								</thead>
								<tbody>
									<tr v-for="row in gscdata">
										<td class="p-3 bg-stone-100 dark:bg-stone-700 border-2 border-white dark:border-stone-600">{{row.query}}</td>
										<td class="p-3 bg-stone-100 dark:bg-stone-700 border-2 border-white dark:border-stone-600">{{row.clicks}}</td>
										<td class="p-3 bg-stone-100 dark:bg-stone-700 border-2 border-white dark:border-stone-600">{{row.impressions}}</td>
										<td class="p-3 bg-stone-100 dark:bg-stone-700 border-2 border-white dark:border-stone-600">{{ getCtr(row.ctr) }}</td>
										<td class="p-3 bg-stone-100 dark:bg-stone-700 border-2 border-white dark:border-stone-600">{{ Number((row.position).toFixed(1)) }}</td>
									</tr>
								</tbody>
							</table>
						</div>
						<div v-if="serviceaccountmessage" class="bg-rose-500 text-white p-2">{{ serviceaccountmessage }}</div>						
					</div>
				</div>`,
	mounted: function()
	{
		this.getDataFromGSC();
	},
	methods: {
		getCtr(ctr)
		{
			var nctr = Number(ctr) * 100;

			return Number((nctr).toFixed(1));
		},
		getDataFromGSC()
		{
			this.disabled = true;

			var selfseo = this;

			tmaxios.get('/api/v1/gscdata',{
				params: {
					'url':			data.urlinfo.route,
					'country': 		this.country,
					'startdate': 	this.startdate,
					'enddate': 		this.enddate,
					'page':  		this.item.urlAbs
				}
			})
			.then(function (response)
			{
				selfseo.disabled 		= false;
				selfseo.serivceaccount 	=  response.data.serivceaccount;
				selfseo.gscdata 		=  response.data.gscdata;

				selfseo.startdate 		= response.data.params.startdate;
				selfseo.enddate 		= response.data.params.enddate;
				selfseo.country 		= response.data.params.country;
			})
			.catch(function (error)
			{
				selfseo.disabled 		= false;

				if(error.response)
				{
					if(error.response.data.serviceaccount != undefined)
					{
						selfseo.serviceaccount = error.response.data.serviceaccount;
					}
					if(error.response.data.gscdata != undefined)
					{
						selfseo.gscdata = error.response.data.gscdata;
					}

					let message = handleErrorMessage(error);
					if(message)
					{
						selfseo.serviceaccountmessage = message;
					}
				}
			});
		},
		saveServiceAccountKey()
		{
			if(this.keyfile == '')
			{
				return false;
			}

			this.disabled 				= true;
			this.errors 				= false;
			this.serviceaccountmessage 	= false;

			var self = this;

			tmaxios.post('/api/v1/gsckeyfile',{
				'url':			data.urlinfo.route,
				'keyfile':  	this.keyfile,
				'country': 		this.country,
				'startdate': 	this.startdate,
				'enddate': 		this.enddate,
				'page':  		this.item.urlAbs
			})
			.then(function (response) {
	
				self.disabled 			= false;

				self.serviceaccount 	= response.data.serviceaccount;
				self.gscdata 			= response.data.gscdata;

				self.startdate 			= response.data.params.startdate;
				self.enddate 			= response.data.params.enddate;
				self.country 			= response.data.params.country;
			})
			.catch(function (error)
			{
				self.disabled 			= false;

				if(error.response)
				{
					if(error.response.data.serviceaccount != undefined)
					{
						self.serviceaccount = error.response.data.serviceaccount;
					}
					if(error.response.data.gscdata != undefined)
					{
						self.gscdata = error.response.data.gscdata;
					}

					let message = handleErrorMessage(error);
					if(message)
					{
						self.serviceaccountmessage = message;
					}
				}
			});
		},
	}
})