<?php

namespace Plugins\seo;

use Psr\Http\Message\ServerRequestInterface as Request;
use Psr\Http\Message\ResponseInterface as Response;
use \Typemill\Plugin;
use \Typemill\Models\StorageWrapper;
use \Typemill\Models\Validation;
use \HannesKirsman\GoogleSearchConsole\SearchConsoleApi;
require_once 'vendor/autoload.php';

class seo extends Plugin
{
	protected $item;

	protected $serviceAccount = true;

	protected $message = false;

	public static function setPremiumLicence()
	{
		return 'MAKER';
	}	
	
    public static function getSubscribedEvents()
    {
		return array(
			'onTwigLoaded' 			=> 'onTwigLoaded',
		);
    }

	public static function addNewRoutes()
	{
		return [
			[
				'httpMethod' 	=> 'post', 
				'route' 		=> '/api/v1/seometa', 
				'name' 			=> 'seometa.store', 
				'class' 		=> 'Plugins\Seo\Seo:storeSeoMeta', 
				'resource' 		=> 'content', 
				'privilege' 	=> 'create'
			],
			[	
				'httpMethod' 	=> 'get', 
				'route' 		=> '/api/v1/gscdata', 
				'name' 			=> 'gscdata.get', 
				'class' 		=> 'Plugins\Seo\Seo:getDataFromGSC', 
				'resource' 		=> 'content', 
				'privilege' 	=> 'create'
			],
			[
				'httpMethod' 	=> 'post', 
				'route' 		=> '/api/v1/gsckeyfile', 
				'name' 			=> 'gsckeyfile.store', 
				'class' 		=> 'Plugins\Seo\Seo:storeKeyfileForGSC', 
				'resource' 		=> 'system', 
				'privilege' 	=> 'view'
			],
		];
	}

   	public function onTwigLoaded()
    {
    	if($this->editorroute)
    	{
		    $pluginSettings = $this->getPluginSettings();
		    $titleAddition 	= $pluginSettings['metatitleaddition'] ?? '';
		    $inlinejs 		= 'var titleAddition = "' . $titleAddition . '";';
	    	
	    	$this->addInlineJS($inlinejs);
	        $this->addJS('/seo/js/seo.js');
    	}
    }

	# store meta title and description
	public function storeSeoMeta(Request $request, Response $response, $args)
	{
		$params 		= $request->getParsedBody();
		
		$item 			= $params['item'] ?? false;
		$data 			= $params['data'] ?? false;
		$settings 		= $this->getSettings();

		# create validation object
		$validation		= new Validation();

		# return standard valitron object for standardfields
		$v = $validation->returnValidator($data);

		$v->rule('required', ['title','description']);
		$v->rule('noHTML', 'title');
		$v->rule('noHTML', 'description');
		$v->rule('lengthBetween', 'title', 5, 65)->message("Length between 5 - 65");
		$v->rule('lengthBetween', 'description', 5, 160)->message("Length between 5 - 160");

		if(!$v->validate())
		{
			$response->getBody()->write(json_encode([
				'message' 	=> 'Please correct the errors in the form.',
				'errors' 	=> $v->errors(),
			]));

			return $response->withHeader('Content-Type', 'application/json')->withStatus(422);
		}

		# get the metadata from page
		$storage 		= new StorageWrapper('\Typemill\Models\Storage');
		$meta 			= $storage->getYaml('contentFolder', '', $item['pathWithoutType'] . '.yaml');

		$meta['meta']['title'] 			= $data['title'];
		$meta['meta']['description'] 	= $data['description'];

		# store the metadata for page 
		$storage->updateYaml('contentFolder', '', $item['pathWithoutType'] . '.yaml', $meta);

		$response->getBody()->write(json_encode([
			'metadata' 			=> $meta
		]));

		return $response->withHeader('Content-Type', 'application/json');
	}

	# store the authentication file for google webmaster tools and return api-data
	public function storeKeyfileForGSC(Request $request, Response $response, $args)
	{
		$params 		= $request->getParsedBody();
		
		$keyfile 		= $params['keyfile'];

		$keyfileArray	= json_decode($keyfile,true);

		if(!$keyfileArray or !is_array($keyfileArray))
		{
			$response->getBody()->write(json_encode([
				'message' 	=> 'Enter a valid Service Account Key in JSON-Format.'
			]));

			return $response->withHeader('Content-Type', 'application/json')->withStatus(422);
		}

		# create validation object
		$validation	= new Validation();

		# return standard valitron object for standardfields
		$v = $validation->returnValidator($keyfileArray);

		$v->rule('required', [
			'type',
			'project_id',
			'private_key_id',
			'private_key',
			'client_email',
			'client_id',
			'project_id',
			'auth_uri',
			'token_uri',
			'auth_provider_x509_cert_url',
			'client_x509_cert_url'
		]);

		if(!$v->validate())
		{
			$errors = $v->errors();
			$response->getBody()->write(json_encode([
				'message' 	=> array_values($errors)[0]
			]));

			return $response->withHeader('Content-Type', 'application/json')->withStatus(422);
		}

		$storedFile = $this->storePluginData('service-account.json', $keyfile);

		if(!$storedFile)
		{
			$response->getBody()->write(json_encode([
				'message' 	=> 'We could not store the service account file.'
			]));

			return $response->withHeader('Content-Type', 'application/json')->withStatus(422);
		}

		# Make Call to google with params
		$pluginSettings = $this->getPluginSettings();

		$params 		= $this->setParams($params, $pluginSettings);

		if(!$params)
		{
			$response->getBody()->write(json_encode([
				'message' 			=> $this->message,
				'serviceaccount' 	=> $this->serviceAccount,
				'gscdata' 			=> $this->serviceAccount # true or false
			]));

			return $response->withHeader('Content-Type', 'application/json')->withStatus(422);
		}

		$rows 			= $this->callApiFromGSC($params, $pluginSettings);

		if($rows === false)
		{
			$response->getBody()->write(json_encode([
				'message' 			=> $this->message,
				'serviceaccount' 	=> $this->serviceAccount,
				'gscdata' 			=> $this->serviceAccount # true or false
			]));

			return $response->withHeader('Content-Type', 'application/json')->withStatus(422);
		}

		$response->getBody()->write(json_encode([
			'gscdata' 			=> $rows, 
			'params' 			=> $params,
			'serviceaccount' 	=> $this->serviceAccount
		]));

		return $response->withHeader('Content-Type', 'application/json');
	}

	public function getDataFromGSC(Request $request, Response $response, $args)
	{
		# get the query params
		$params 			= $request->getQueryParams();
		$pluginSettings 	= $this->getPluginSettings();

		$params 			= $this->setParams($params, $pluginSettings);

		if(!$params)
		{
			$response->getBody()->write(json_encode([
				'message' 	=> 'No parameters found, refresh the page and try again.'
			]));

			return $response->withHeader('Content-Type', 'application/json')->withStatus(422);
		}

		$rows 				= $this->callApiFromGSC($params, $pluginSettings);

		if($rows === false)
		{
			$response->getBody()->write(json_encode([
				'message' 			=> $this->message,
				'serviceaccount' 	=> $this->serviceAccount,
				'gscdata' 			=> $this->serviceAccount # true or false
			]));

			return $response->withHeader('Content-Type', 'application/json')->withStatus(422);
		}

		$response->getBody()->write(json_encode([
			'serviceaccount' 	=> $this->serviceAccount,
			'gscdata' 			=> $rows, 
			'params' 			=> $params
		]));

		return $response->withHeader('Content-Type', 'application/json');
	}

	private function setParams($params, $pluginSettings)
	{
		$cleanParams = [];

		# MANDATORY PAGE
		if(!isset($params['page']) OR filter_var($params['page'], \FILTER_VALIDATE_URL) === false)
		{
			$this->message = 'the site_url is missing or not a valid url.';

			return false;
		}
		$cleanParams['page'] = $params['page'];


		# MANDATORY ENDDATE
		$enddate = date('Y-m-d');
		if(isset($params['enddate']) && $params['enddate'] != '' && \DateTime::createFromFormat('Y-m-d', $params['enddate']))
		{
			$enddate = \DateTime::createFromFormat('Y-m-d', $params['enddate']);
			$enddate = $enddate->format('Y-m-d');
		}
		$cleanParams['enddate'] = $enddate;


		# MANDATORY STARTDATE
		$startdate = date('Y-m-d', strtotime("-3 month"));
		if(isset($params['startdate']) && $params['startdate'] != '' && \DateTime::createFromFormat('Y-m-d', $params['startdate']))
		{
			$startdate = \DateTime::createFromFormat('Y-m-d', $params['startdate']);
			$startdate = $startdate->format('Y-m-d');
		}
		elseif(isset($pluginSettings['gscperiod']) && is_int($pluginSettings['gscperiod']))
		{
			$period = $pluginSettings['gscperiod'];
			$startdate = date('Y-m-d', strtotime("-$period month"));
		}
		$cleanParams['startdate'] = $startdate;

		# OPTIONAL COUNTRY
		if(isset($params['country']) && $params['country'] && preg_match('/^[A-Z]{3}$/', $params['country']) )
		{
			$cleanParams['country'] = $params['country'];
		}
		elseif(isset($pluginSettings['gsccountry']) && preg_match('/^[A-Z]{3}$/', $pluginSettings['gsccountry']))
		{
			$cleanParams['country'] = $pluginSettings['gsccountry'];
		}

		return $cleanParams;
	}

	private function callApiFromGSC($params, $pluginSettings)
	{
		if(!isset($pluginSettings['site_url']))
		{
			$this->message = 'Please add a valid site url/google property into the plugin settings.';

			return false;
		}

		if(filter_var($pluginSettings['site_url'], \FILTER_VALIDATE_URL) === false)
		{
			$this->message = 'The site url/google property in the plugin settings is not a valid url.';

			return false;
		}

		$authJson = $this->getPluginData('service-account.json');
		if(!$authJson)
		{
			$this->serviceAccount = false;

			$this->message = 'service-account.json not found.';

			return false;
		}
                       
		# decode as an array
		$authJson = json_decode($authJson, true);

		$searchConsole = new SearchConsoleApi($authJson);

		$filters = [
			[
		        'dimension'   => 'page',
		        'operator'    => 'equals',
		        'expression'  => $params['page'],
			]
		];

		if(isset($params['country']))
		{
			$filters[] =
		     [
		        'dimension'   => 'country',
		        'operator'    => 'equals',
		        'expression'  => $params['country'],
		     ];
		}

		$options = SearchConsoleApi::getDefaultOptions();
		$options['site_url']    = $pluginSettings['site_url'];
		$options['start_date']  = $params['startdate'];
		$options['end_date']    = $params['enddate'];
  		$options['dimensions']	= ['query'];
		$options['setDimensionFilterGroups'] = [
		  'filters' => $filters
		];


		$rows = $searchConsole->getRows($options);

		# check for errors
		if ($rows instanceof \Google\Service\Exception)
		{
		    $errorMessage = $rows->getMessage();
		    $errorDetails = json_decode($errorMessage, true);

		    if (isset($errorDetails['error']['message']))
		    {
		        $this->message = $errorDetails['error']['message'];
		    }
		    else
		    {
		        $this->message = "The Google API returned an error, but the message was not found.";
		    }

		    return false;
		}

		# prepare rows for frontend display
		$datarows = [];
		foreach($rows as $row)
		{
			$datarows[] = [
				'clicks' 		=> $row->clicks,
				'ctr'			=> $row->ctr,
				'impressions'	=> $row->impressions,
				'position'		=> $row->position,
				'query'			=> $row->keys[0]
			];
		}

		return $datarows;
	}
}